==> bodo-2022.5.8-cp310-cp310-macosx_10_15_x86_64.whl: bodo/hiframes/series_kernels.py <==
"""Some kernels for Series related functions. This is a legacy file that needs to be
refactored.
"""
import datetime
import numba
import numpy as np
from numba.core import types
from numba.extending import overload, register_jitable
import bodo
from bodo.libs.int_arr_ext import IntDtype
from bodo.utils.typing import decode_if_dict_array


def _column_filter_impl(B, ind):
    uke__efdtb = bodo.hiframes.rolling.alloc_shift(len(B), B, (-1,))
    for ynpmk__tfx in numba.parfors.parfor.internal_prange(len(uke__efdtb)):
        if ind[ynpmk__tfx]:
            uke__efdtb[ynpmk__tfx] = B[ynpmk__tfx]
        else:
            bodo.libs.array_kernels.setna(uke__efdtb, ynpmk__tfx)
    return uke__efdtb


@numba.njit(no_cpython_wrapper=True)
def _series_dropna_str_alloc_impl_inner(B):
    B = decode_if_dict_array(B)
    jlvf__vodqt = len(B)
    mpajv__uti = 0
    for ynpmk__tfx in range(len(B)):
        if bodo.libs.str_arr_ext.str_arr_is_na(B, ynpmk__tfx):
            mpajv__uti += 1
    ngx__tjll = jlvf__vodqt - mpajv__uti
    gtpuk__dgvbx = bodo.libs.str_arr_ext.num_total_chars(B)
    uke__efdtb = bodo.libs.str_arr_ext.pre_alloc_string_array(ngx__tjll,
        gtpuk__dgvbx)
    bodo.libs.str_arr_ext.copy_non_null_offsets(uke__efdtb, B)
    bodo.libs.str_arr_ext.copy_data(uke__efdtb, B)
    bodo.libs.str_arr_ext.set_null_bits_to_value(uke__efdtb, -1)
    return uke__efdtb


def _get_nan(val):
    return np.nan


@overload(_get_nan, no_unliteral=True)
def _get_nan_overload(val):
    if isinstance(val, (types.NPDatetime, types.NPTimedelta)):
        nat = val('NaT')
        return lambda val: nat
    if isinstance(val, types.Float):
        return lambda val: np.nan
    return lambda val: val


def _get_type_max_value(dtype):
    return 0


@overload(_get_type_max_value, inline='always', no_unliteral=True)
def _get_type_max_value_overload(dtype):
    if isinstance(dtype, (bodo.IntegerArrayType, IntDtype)):
        _dtype = dtype.dtype
        return lambda dtype: numba.cpython.builtins.get_type_max_value(_dtype)
    if dtype == bodo.datetime_date_array_type:
        return lambda dtype: _get_date_max_value()
    if isinstance(dtype.dtype, types.NPDatetime):
        return lambda dtype: bodo.hiframes.pd_timestamp_ext.integer_to_dt64(
            numba.cpython.builtins.get_type_max_value(numba.core.types.int64))
    if isinstance(dtype.dtype, types.NPTimedelta):
        return (lambda dtype: bodo.hiframes.pd_timestamp_ext.
            integer_to_timedelta64(numba.cpython.builtins.
            get_type_max_value(numba.core.types.int64)))
    if dtype.dtype == types.bool_:
        return lambda dtype: True
    return lambda dtype: numba.cpython.builtins.get_type_max_value(dtype)


@register_jitable
def _get_date_max_value():
    return datetime.date(datetime.MAXYEAR, 12, 31)


def _get_type_min_value(dtype):
    return 0


@overload(_get_type_min_value, inline='always', no_unliteral=True)
def _get_type_min_value_overload(dtype):
    if isinstance(dtype, (bodo.IntegerArrayType, IntDtype)):
        _dtype = dtype.dtype
        return lambda dtype: numba.cpython.builtins.get_type_min_value(_dtype)
    if dtype == bodo.datetime_date_array_type:
        return lambda dtype: _get_date_min_value()
    if isinstance(dtype.dtype, types.NPDatetime):
        return lambda dtype: bodo.hiframes.pd_timestamp_ext.integer_to_dt64(
            numba.cpython.builtins.get_type_min_value(numba.core.types.int64))
    if isinstance(dtype.dtype, types.NPTimedelta):
        return (lambda dtype: bodo.hiframes.pd_timestamp_ext.
            integer_to_timedelta64(numba.cpython.builtins.
            get_type_min_value(numba.core.types.uint64)))
    if dtype.dtype == types.bool_:
        return lambda dtype: False
    return lambda dtype: numba.cpython.builtins.get_type_min_value(dtype)


@register_jitable
def _get_date_min_value():
    return datetime.date(datetime.MINYEAR, 1, 1)


@overload(min)
def indval_min(a1, a2):
    if a1 == types.bool_ and a2 == types.bool_:

        def min_impl(a1, a2):
            if a1 > a2:
                return a2
            return a1
        return min_impl


@overload(max)
def indval_max(a1, a2):
    if a1 == types.bool_ and a2 == types.bool_:

        def max_impl(a1, a2):
            if a2 > a1:
                return a2
            return a1
        return max_impl


@numba.njit
def _sum_handle_nan(s, count):
    if not count:
        s = bodo.hiframes.series_kernels._get_nan(s)
    return s


@numba.njit
def _box_cat_val(s, cat_dtype, count):
    if s == -1 or count == 0:
        return bodo.hiframes.series_kernels._get_nan(cat_dtype.categories[0])
    return cat_dtype.categories[s]


@numba.generated_jit
def get_float_nan(s):
    nan = np.nan
    if s == types.float32:
        nan = np.float32('nan')
    return lambda s: nan


@numba.njit
def _mean_handle_nan(s, count):
    if not count:
        s = get_float_nan(s)
    else:
        s = s / count
    return s


@numba.njit
def _var_handle_mincount(s, count, min_count):
    if count < min_count:
        res = np.nan
    else:
        res = s
    return res


@numba.njit
def _compute_var_nan_count_ddof(first_moment, second_moment, count, ddof):
    if count == 0 or count <= ddof:
        s = np.nan
    else:
        s = second_moment - first_moment * first_moment / count
        s = s / (count - ddof)
    return s


@numba.njit
def _sem_handle_nan(res, count):
    if count < 1:
        jzhnl__hfet = np.nan
    else:
        jzhnl__hfet = (res / count) ** 0.5
    return jzhnl__hfet


@numba.njit
def lt_f(a, b):
    return a < b


@numba.njit
def gt_f(a, b):
    return a > b


@numba.njit
def compute_skew(first_moment, second_moment, third_moment, count):
    if count < 3:
        return np.nan
    oknz__ftgr = first_moment / count
    xjaqp__wkq = (third_moment - 3 * second_moment * oknz__ftgr + 2 * count *
        oknz__ftgr ** 3)
    hah__rgc = second_moment - oknz__ftgr * first_moment
    s = count * (count - 1) ** 1.5 / (count - 2) * xjaqp__wkq / hah__rgc ** 1.5
    s = s / (count - 1)
    return s


@numba.njit
def compute_kurt(first_moment, second_moment, third_moment, fourth_moment,
    count):
    if count < 4:
        return np.nan
    oknz__ftgr = first_moment / count
    xozr__roycy = (fourth_moment - 4 * third_moment * oknz__ftgr + 6 *
        second_moment * oknz__ftgr ** 2 - 3 * count * oknz__ftgr ** 4)
    kydi__fom = second_moment - oknz__ftgr * first_moment
    enhv__lio = 3 * (count - 1) ** 2 / ((count - 2) * (count - 3))
    ita__imfi = count * (count + 1) * (count - 1) * xozr__roycy
    rpv__fiu = (count - 2) * (count - 3) * kydi__fom ** 2
    s = (count - 1) * (ita__imfi / rpv__fiu - enhv__lio)
    s = s / (count - 1)
    return s

==> bodo-2022.7.1-cp310-cp310-macosx_10_15_x86_64.whl: bodo/hiframes/series_kernels.py <==
"""Some kernels for Series related functions. This is a legacy file that needs to be
refactored.
"""
import datetime
import numba
import numpy as np
from numba.core import types
from numba.extending import overload, register_jitable
import bodo
from bodo.libs.int_arr_ext import IntDtype
from bodo.utils.typing import decode_if_dict_array


def _column_filter_impl(B, ind):
    lzstt__yhv = bodo.hiframes.rolling.alloc_shift(len(B), B, (-1,))
    for dmi__offb in numba.parfors.parfor.internal_prange(len(lzstt__yhv)):
        if ind[dmi__offb]:
            lzstt__yhv[dmi__offb] = B[dmi__offb]
        else:
            bodo.libs.array_kernels.setna(lzstt__yhv, dmi__offb)
    return lzstt__yhv


@numba.njit(no_cpython_wrapper=True)
def _series_dropna_str_alloc_impl_inner(B):
    B = decode_if_dict_array(B)
    gyzmu__wpe = len(B)
    aeclk__jbwlt = 0
    for dmi__offb in range(len(B)):
        if bodo.libs.str_arr_ext.str_arr_is_na(B, dmi__offb):
            aeclk__jbwlt += 1
    kgfep__trzvb = gyzmu__wpe - aeclk__jbwlt
    mvjx__kgkx = bodo.libs.str_arr_ext.num_total_chars(B)
    lzstt__yhv = bodo.libs.str_arr_ext.pre_alloc_string_array(kgfep__trzvb,
        mvjx__kgkx)
    bodo.libs.str_arr_ext.copy_non_null_offsets(lzstt__yhv, B)
    bodo.libs.str_arr_ext.copy_data(lzstt__yhv, B)
    bodo.libs.str_arr_ext.set_null_bits_to_value(lzstt__yhv, -1)
    return lzstt__yhv


def _get_nan(val):
    return np.nan


@overload(_get_nan, no_unliteral=True)
def _get_nan_overload(val):
    if isinstance(val, (types.NPDatetime, types.NPTimedelta)):
        nat = val('NaT')
        return lambda val: nat
    if isinstance(val, types.Float):
        return lambda val: np.nan
    return lambda val: val


def _get_type_max_value(dtype):
    return 0


@overload(_get_type_max_value, inline='always', no_unliteral=True)
def _get_type_max_value_overload(dtype):
    if isinstance(dtype, (bodo.IntegerArrayType, IntDtype)):
        _dtype = dtype.dtype
        return lambda dtype: numba.cpython.builtins.get_type_max_value(_dtype)
    if dtype == bodo.datetime_date_array_type:
        return lambda dtype: _get_date_max_value()
    if isinstance(dtype.dtype, types.NPDatetime):
        return lambda dtype: bodo.hiframes.pd_timestamp_ext.integer_to_dt64(
            numba.cpython.builtins.get_type_max_value(numba.core.types.int64))
    if isinstance(dtype.dtype, types.NPTimedelta):
        return (lambda dtype: bodo.hiframes.pd_timestamp_ext.
            integer_to_timedelta64(numba.cpython.builtins.
            get_type_max_value(numba.core.types.int64)))
    if dtype.dtype == types.bool_:
        return lambda dtype: True
    return lambda dtype: numba.cpython.builtins.get_type_max_value(dtype)


@register_jitable
def _get_date_max_value():
    return datetime.date(datetime.MAXYEAR, 12, 31)


def _get_type_min_value(dtype):
    return 0


@overload(_get_type_min_value, inline='always', no_unliteral=True)
def _get_type_min_value_overload(dtype):
    if isinstance(dtype, (bodo.IntegerArrayType, IntDtype)):
        _dtype = dtype.dtype
        return lambda dtype: numba.cpython.builtins.get_type_min_value(_dtype)
    if dtype == bodo.datetime_date_array_type:
        return lambda dtype: _get_date_min_value()
    if isinstance(dtype.dtype, types.NPDatetime):
        return lambda dtype: bodo.hiframes.pd_timestamp_ext.integer_to_dt64(
            numba.cpython.builtins.get_type_min_value(numba.core.types.int64))
    if isinstance(dtype.dtype, types.NPTimedelta):
        return (lambda dtype: bodo.hiframes.pd_timestamp_ext.
            integer_to_timedelta64(numba.cpython.builtins.
            get_type_min_value(numba.core.types.uint64)))
    if dtype.dtype == types.bool_:
        return lambda dtype: False
    return lambda dtype: numba.cpython.builtins.get_type_min_value(dtype)


@register_jitable
def _get_date_min_value():
    return datetime.date(datetime.MINYEAR, 1, 1)


@overload(min)
def indval_min(a1, a2):
    if a1 == types.bool_ and a2 == types.bool_:

        def min_impl(a1, a2):
            if a1 > a2:
                return a2
            return a1
        return min_impl


@overload(max)
def indval_max(a1, a2):
    if a1 == types.bool_ and a2 == types.bool_:

        def max_impl(a1, a2):
            if a2 > a1:
                return a2
            return a1
        return max_impl


@numba.njit
def _sum_handle_nan(s, count):
    if not count:
        s = bodo.hiframes.series_kernels._get_nan(s)
    return s


@numba.njit
def _box_cat_val(s, cat_dtype, count):
    if s == -1 or count == 0:
        return bodo.hiframes.series_kernels._get_nan(cat_dtype.categories[0])
    return cat_dtype.categories[s]


@numba.generated_jit
def get_float_nan(s):
    nan = np.nan
    if s == types.float32:
        nan = np.float32('nan')
    return lambda s: nan


@numba.njit
def _mean_handle_nan(s, count):
    if not count:
        s = get_float_nan(s)
    else:
        s = s / count
    return s


@numba.njit
def _var_handle_mincount(s, count, min_count):
    if count < min_count:
        res = np.nan
    else:
        res = s
    return res


@numba.njit
def _compute_var_nan_count_ddof(first_moment, second_moment, count, ddof):
    if count == 0 or count <= ddof:
        s = np.nan
    else:
        s = second_moment - first_moment * first_moment / count
        s = s / (count - ddof)
    return s


@numba.njit
def _sem_handle_nan(res, count):
    if count < 1:
        lvwv__nfsj = np.nan
    else:
        lvwv__nfsj = (res / count) ** 0.5
    return lvwv__nfsj


@numba.njit
def lt_f(a, b):
    return a < b


@numba.njit
def gt_f(a, b):
    return a > b


@numba.njit
def compute_skew(first_moment, second_moment, third_moment, count):
    if count < 3:
        return np.nan
    vyzy__rtopy = first_moment / count
    rqu__qebh = (third_moment - 3 * second_moment * vyzy__rtopy + 2 * count *
        vyzy__rtopy ** 3)
    zicl__dfcze = second_moment - vyzy__rtopy * first_moment
    s = count * (count - 1) ** 1.5 / (count - 2
        ) * rqu__qebh / zicl__dfcze ** 1.5
    s = s / (count - 1)
    return s


@numba.njit
def compute_kurt(first_moment, second_moment, third_moment, fourth_moment,
    count):
    if count < 4:
        return np.nan
    vyzy__rtopy = first_moment / count
    kfer__tcovk = (fourth_moment - 4 * third_moment * vyzy__rtopy + 6 *
        second_moment * vyzy__rtopy ** 2 - 3 * count * vyzy__rtopy ** 4)
    zyvu__xcbc = second_moment - vyzy__rtopy * first_moment
    koffw__tfful = 3 * (count - 1) ** 2 / ((count - 2) * (count - 3))
    dmbkk__inz = count * (count + 1) * (count - 1) * kfer__tcovk
    tnppa__unlmy = (count - 2) * (count - 3) * zyvu__xcbc ** 2
    s = (count - 1) * (dmbkk__inz / tnppa__unlmy - koffw__tfful)
    s = s / (count - 1)
    return s
